/*****************************************************************************/
/*****************************************************************************/
/**                                GETOPT.C                                 **/
/**       Fonction d'analyse des paramtres des commandes (Style UNIX)      **/
/**                                                                         **/
/**   Dernire modification : 07/02/91                                      **/
/**     Ajout de la selection du caractre d'indication d'option par 'SW'   **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*                              Description                                  */
/*                                                                           */
/*  Cette fonction permet d'analyser les paramtres optionnels               */
/*  d'une commande MS/DOS dans le style utilis par UNIX.                    */
/*                                                                           */
/*      commande [-options] [oprandes...]                                   */
/*                                                                           */
/*   par exemple la commande :                                               */
/*                                                                           */
/*        dircat  [-d] [-s] [-w width]  file1 [file2...]                     */
/*                                                                           */
/*   comporte 3 options : d, s, et w. (cette derniere option comporte en     */
/*     outre un argument). Ces options peuvent tre tapes dans n'importe    */
/*     quel ordre ou tre groupes :                                         */
/*                                                                           */
/*        dircat  -s -w 15 -d  toto.txt                                      */
/*        dircat  -ds  toto.txt titi.txt                                     */
/*                                                                           */
/*   UNIX System V admet les rgles suivantes pour la syntaxe des commandes: */
/*                                                                           */
/*   Rgle 1  : La longueur des noms de commandes doit tre comprise entre   */
/*              2 et 9 caractres.     - non valable pour MS/DOS (1 et 8) -  */
/*   Rgle 2  : Les noms de commandes ne doivent inclure que des lettres     */
/*              minuscules et des chiffres.     - non valable pour MS/DOS -  */
/*   Rgle 3  : Les noms d'option doivent avoir une longueur d'un caractre  */
/*   Rgle 4  : Toutes les options doivent commencer par le dlimiteur '-'   */
/*   Rgle 5  : Les options sans arguments peuvent tre groupes derrire    */
/*              le mme dlimiteur.                                          */
/*   Rgle 6  : Le premier argument d'option suivant une option doit tre    */
/*              prcde par un espace.                                      */
/*   Rgle 7  : Les arguments d'option ne peuvent pas tre optionnels        */
/*   Rgle 8  : Les groupes d'arguments d'option suivant une option          */
/*              doivent tre spars par des virgules, ou spars par des    */
/*              espaces et entours par des apostrophes.                     */
/*   Rgle 9  : Toutes les options doivent prcder les oprandes dans la    */
/*              ligne de commande.                                           */
/*   Rgle 10 : Les caractres '--' peuvent tre utiliss pour dlimiter     */
/*              la fin des options.                                          */
/*   Rgle 11 : L'ordre des options est indifrent.                          */
/*   Rgle 12 : L'ordre des oprandes et leur interprtation doit tre       */
/*              dtermin par une analyse spcifique dans la commande.       */
/*   Rgle 13 : Le caractre '-' prcd et suivi par un espace doit         */
/*              tre utilis seulement pour spcifier l'entre standard.     */
/*                                                                           */
/*   La fonction 'getopt', incluse dans l'analyse des paramtres de la       */
/*   commande, permet de tenir compte de ces rgles et de simplifier         */
/*   cette analyse.                                                          */
/*                                                                           */
/*   'getopt' retourne la lettre d'option suivante, contenue dans 'argv',    */
/*   qui correspond  'optstring'. Quand toutes les options ont t          */
/*   dcodes, la fonction retourne EOF. Si une option inconnue est          */
/*   rencontre, la fonction retourne '?'.                                   */
/*                                                                           */
/*   'optarg' contient (si l'option le permet) l'argument correspondant     */
/*   l'option rencontre.                                                    */
/*                                                                           */
/*   'optind' contient l'index de 'argv' correspondant  la prochaine        */
/*   option  dcoder, ou  la premire oprande aprs la fin de l'analyse   */
/*   des options.                                                            */
/*                                                                           */
/*   'optstring' est une chaine de caractres contenant les lettres des      */
/*   options admises. Si une lettre est suivie de ':', l'option doit         */
/*   comporter un argument.                                                  */
/*                                                                           */
/*   Exemple :                                                               */
/*                                                                           */
/*   Analyse de la commande :  comm [-a] [-b] [-o file] extfile...           */
/*                                                                           */
/*      #include "getopt.c"                                                  */
/*                                                                           */
/*      main(int argc, char *argv[])                                         */
/*          {                                                                */
/*           int opta, optb, opto;                                           */
/*           char ofile[40];                                                 */
/*           char c;                                                         */
/*           int errflag;                                                    */
/*                                                                           */
/*           opta=0;                     / initialisation des options  0 /  */
/*           optb=0;                                                         */
/*           opto=0;                                                         */
/*                                                                           */
/*           while((c=getopt(argc, argv, "ao:b")) != EOF)                    */
/*             {                                                             */
/*              switch(c)                         / selection des options /  */
/*                {                                                          */
/*                 case 'a' : opta = 1;                                      */
/*                            break;                                         */
/*                                                                           */
/*                 case 'b' : optb = 1;                                      */
/*                            break;                                         */
/*                                                                           */
/*                 case 'o' : opto = 1;                                      */
/*                            strcpy(ofile, optarg);                         */
/*                            break;                                         */
/*                                                                           */
/*                 case '?' : errflag++;                                     */
/*                            break;                                         */
/*                }                                                          */
/*             }                                                             */
/*                                                                           */
/*           if(errflag)                         / sortie en cas d'erreur /  */
/*             {                                                             */
/*              fprintf(stderr, "usage : comm [-a] [-b] [-o file] extfile"); */
/*              exit(-1);                                                    */
/*             }                                                             */
/*                                                                           */
/*           for(; optind<argc; optind++)         / analyse des oprandes /  */
/*             {                                                             */
/*              if(access(argv[optind], 4)                                   */
/*                {                                                          */
/*                .                                                          */
/*                .                                                          */
/*                .                                                          */
/*          }                                                                */
/*                                                                           */
/*   Divergences de la fonction incluse dans ce listing :                    */
/*                                                                           */
/*     1 - Il est possible de changer le caractre dlimiteur '-' par        */
/*         un autre caractre (par exemple '/', plus utilis par MS/DOS)     */
/*         en placant ce caractre dans la variable 'SW' avant l'appel       */
/*         de 'getopt'.                                                      */
/*     2 - La rgle 13 n'est pas implante.                                  */
/*     3 - Les versions actuelles d'UNIX ne satisfont pas aux rgles 5 et 6. */
/*         En effet il est actuellement possible de taper :                  */
/*                             cmd -abo xxx                                  */
/*               au lieu de    cmd -ab -o xxx     (violation rgle 5)        */
/*         ou                  cmd -ab -oxxx                                 */
/*               au lieu de    cmd -ab -o xxx     (violation rgle 6)        */
/*         Cette facon d'entrer les paramtres pouvant ne plus tre support */
/*         dans une prochaine version d'UNIX. Cepandant, afin de ne pas      */
/*         changer les (mauvaises !) habitudes, ce type de syntaxe          */
/*         t implant dans ce listing.                                     */
/*                                                                           */
/*                                                                           */
#include <string.h>
#include <stdio.h>

char *optarg;
int optind=1;
int SW = '-';

int getopt(int argc, char *argv[], char *optstring)
   {
    char *ptopt;
    int retour='?';
    int len_arg;

    optarg=NULL;

    if(optind == argc)                     /* si fin de la ligne de commande */
       return(EOF);

    if(*argv[optind] == SW)                 /* si parametre commence par '-' */
      {
       len_arg=strlen(argv[optind]);            /* calcul longueur parametre */
       retour=argv[optind][1];          /* acquisition du caractre d'option */
       if(retour == SW)                                    /* si option '--' */
         {
          optind++;                           /* selection parametre suivant */
          return(EOF);                         /* fin analyse des paramtres */
         }
       ptopt=strchr(optstring, retour);   /* recherche option dans optstring */
       if(ptopt == NULL)                            /* si option non trouve */
         {
          optind++;
          return('?');                                 /* sortie avec erreur */
         }
       if(*(ptopt+1) == ':')        /* si option suivi de ':' dans optstring */
         {
          if(len_arg == 2)        /* si option entre dans la forme '-o xxx' */
            {
             optind++;                                 /* acces  l'argument */
             optarg=argv[optind];                    /* acquisition argument */
                                       /* si option non suivie d'un argument */
             if((optind>=argc) || (*optarg=='-')) 
               {
                optind++;
                return('?');                           /* sortie avec erreur */
               }
            }
          else                     /* si option entre sous la forme '-oxxx' */
            {
             optarg=argv[optind]+2;                  /* acquisition argument */
            }
         }
       else          /* si caractere option non suivie de ':' dans optstring */                
         {
          if(len_arg != 2)                   /* si options groupes ('-abc') */
            {
             argv[optind]++;           /* prparation du passage suivant ... */
             *argv[optind]=SW;        /* ... en transformant '-abc' en '-bc' */
             optind--;
            }
         }
      }
    else                             /* si le paramtre n'est pas une option */
      {
       return(EOF);                       /* fin de l'analyse des paramtres */
      }
    optind++;                                 /* selection parametre suivant */
    return (retour);                         /* retour du caractre d'option */
   }
